@@ -2,7 +2,7 @@
    Copyright (c) 2002-2007 Darren Salt, Fredrick Meunier
    Based on tap.c, copyright (c) 2001 Philip Kendall
 
-   $Id: csw.c 3708 2008-07-01 08:07:01Z pak21 $
+   $Id: csw.c 4780 2012-11-29 19:09:52Z sbaldovi $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,7 +39,6 @@ libspectrum_csw_read( libspectrum_tape *tape,
 {
   libspectrum_tape_block *block = NULL;
   libspectrum_tape_rle_pulse_block *csw_block;
-  libspectrum_error error;
 
   int compressed;
 
@@ -86,8 +85,8 @@ libspectrum_csw_read( libspectrum_tape *tape,
 
     if( compressed != 0 && compressed != 1 ) goto csw_bad_compress;
 
+    if( length < 29 - buffer[12] ) goto csw_short;
     length -= 29 - buffer[12];
-    if( length < 0 ) goto csw_short;
     buffer += 29 + buffer[12];
 
     break;
@@ -112,6 +111,8 @@ libspectrum_csw_read( libspectrum_tape *tape,
   if( compressed ) {
     /* Compressed data... */
 #ifdef HAVE_ZLIB_H
+    libspectrum_error error;
+
     csw_block->data = NULL;
     csw_block->length = 0;
     error = libspectrum_zlib_inflate( buffer, length, &csw_block->data,
