@@ -2,7 +2,7 @@
                 routines, but not by user code
    Copyright (c) 2001-2008 Philip Kendall, Darren Salt
 
-   $Id: internals.h 4122 2010-02-21 05:45:00Z fredm $
+   $Id: internals.h 4695 2012-05-07 02:03:10Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -166,6 +166,7 @@ extern const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_SYNC1;
 extern const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_SYNC2;
 extern const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_DATA0;
 extern const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_DATA1;
+extern const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_TAIL;
 
 /* Tape routines */
 
@@ -218,6 +219,10 @@ libspectrum_csw_write( libspectrum_byte **buffer, size_t *length,
 libspectrum_error
 libspectrum_wav_read( libspectrum_tape *tape, const char *filename );
 
+libspectrum_error
+internal_pzx_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
+                   const size_t length );
+
 libspectrum_tape_block*
 libspectrum_tape_block_internal_init(
                                 libspectrum_tape_block_state *iterator,
@@ -235,4 +240,32 @@ libspectrum_sign_data( libspectrum_byte **signature, size_t *signature_length,
 		       libspectrum_byte *data, size_t data_length,
 		       libspectrum_rzx_dsa_key *key );
 
+/* Utility functions */
+
+libspectrum_dword 
+libspectrum_ms_to_tstates( libspectrum_dword ms );
+
+libspectrum_dword 
+libspectrum_tstates_to_ms( libspectrum_dword tstates );
+
+void
+libspectrum_set_pause_ms( libspectrum_tape_block *block,
+                          libspectrum_dword pause_ms );
+
+void
+libspectrum_set_pause_tstates( libspectrum_tape_block *block,
+                               libspectrum_dword pause_tstates );
+
+extern const int LIBSPECTRUM_BITS_IN_BYTE;
+
+/* glib replacement functions */
+
+#ifndef HAVE_LIB_GLIB		/* Only if we are using glib replacement */
+void
+libspectrum_slist_cleanup( void );
+
+void
+libspectrum_hashtable_cleanup( void );
+#endif				/* #ifndef HAVE_LIB_GLIB */
+
 #endif				/* #ifndef LIBSPECTRUM_INTERNALS_H */
