@@ -1,7 +1,7 @@
 /* libspectrum.c: Some general routines
    Copyright (c) 2001-2009 Philip Kendall, Darren Salt, Fredrick Meunier
 
-   $Id: libspectrum.c 4148 2010-08-25 21:20:50Z pak21 $
+   $Id: libspectrum.c 4830 2012-12-30 21:37:36Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -121,6 +121,15 @@ libspectrum_init( void )
   return LIBSPECTRUM_ERROR_NONE;
 }
 
+void
+libspectrum_end( void )
+{
+#ifndef HAVE_LIB_GLIB
+  libspectrum_slist_cleanup();
+  libspectrum_hashtable_cleanup();
+#endif				/* #ifndef HAVE_LIB_GLIB */
+}
+
 #ifdef HAVE_GCRYPT_H
 static void
 gcrypt_log_handler( void *opaque, int level, const char *format, va_list ap )
@@ -228,6 +237,7 @@ libspectrum_machine_name( libspectrum_machine type )
   case LIBSPECTRUM_MACHINE_TC2068:   return "Timex TC2068";
   case LIBSPECTRUM_MACHINE_TS2068:   return "Timex TS2068";
   case LIBSPECTRUM_MACHINE_128:      return "Spectrum 128K";
+  case LIBSPECTRUM_MACHINE_128E:     return "Spectrum 128Ke";
   case LIBSPECTRUM_MACHINE_PLUS2:    return "Spectrum +2";
   case LIBSPECTRUM_MACHINE_PENT:     return "Pentagon 128K";
   case LIBSPECTRUM_MACHINE_PENT512:  return "Pentagon 512K";
@@ -287,7 +297,7 @@ libspectrum_machine_capabilities( libspectrum_machine type )
   switch( type ) {
   case LIBSPECTRUM_MACHINE_128: case LIBSPECTRUM_MACHINE_PLUS2:
   case LIBSPECTRUM_MACHINE_PLUS2A: case LIBSPECTRUM_MACHINE_PLUS3:
-  case LIBSPECTRUM_MACHINE_PLUS3E:
+  case LIBSPECTRUM_MACHINE_PLUS3E: case LIBSPECTRUM_MACHINE_128E:
   case LIBSPECTRUM_MACHINE_TC2068: case LIBSPECTRUM_MACHINE_TS2068:
   case LIBSPECTRUM_MACHINE_PENT:
   case LIBSPECTRUM_MACHINE_PENT512: case LIBSPECTRUM_MACHINE_PENT1024:
@@ -302,7 +312,7 @@ libspectrum_machine_capabilities( libspectrum_machine type )
   switch( type ) {
   case LIBSPECTRUM_MACHINE_128: case LIBSPECTRUM_MACHINE_PLUS2:
   case LIBSPECTRUM_MACHINE_PLUS2A: case LIBSPECTRUM_MACHINE_PLUS3:
-  case LIBSPECTRUM_MACHINE_PLUS3E:
+  case LIBSPECTRUM_MACHINE_PLUS3E: case LIBSPECTRUM_MACHINE_128E:
   case LIBSPECTRUM_MACHINE_PENT:
   case LIBSPECTRUM_MACHINE_PENT512: case LIBSPECTRUM_MACHINE_PENT1024:
   case LIBSPECTRUM_MACHINE_SCORP:
@@ -316,7 +326,7 @@ libspectrum_machine_capabilities( libspectrum_machine type )
   /* +3 Spectrum-style 0x1ffd memory paging */
   switch( type ) {
   case LIBSPECTRUM_MACHINE_PLUS2A: case LIBSPECTRUM_MACHINE_PLUS3:
-  case LIBSPECTRUM_MACHINE_PLUS3E:
+  case LIBSPECTRUM_MACHINE_PLUS3E: case LIBSPECTRUM_MACHINE_128E:
     capabilities |= LIBSPECTRUM_MACHINE_CAPABILITY_PLUS3_MEMORY; break;
   default:
     break;
@@ -370,6 +380,7 @@ libspectrum_machine_capabilities( libspectrum_machine type )
   switch( type ) {
   case LIBSPECTRUM_MACHINE_PLUS2: case LIBSPECTRUM_MACHINE_PLUS2A:
   case LIBSPECTRUM_MACHINE_PLUS3: case LIBSPECTRUM_MACHINE_PLUS3E:
+  case LIBSPECTRUM_MACHINE_128E:
     capabilities |= LIBSPECTRUM_MACHINE_CAPABILITY_SINCLAIR_JOYSTICK; break;
   default:
     break;
@@ -463,10 +474,12 @@ libspectrum_identify_file_with_class(
   error = libspectrum_identify_file_with_class( type, libspectrum_class,
 						new_filename, new_buffer,
 						new_length );
-  if( error ) return error;
 
+  /* new_filename or buffer will be allocated in libspectrum_uncompress_file */
   libspectrum_free( new_filename ); libspectrum_free( new_buffer );
 
+  if( error ) return error;
+
   return LIBSPECTRUM_ERROR_NONE;
 }
 
@@ -524,6 +537,7 @@ libspectrum_identify_file_raw( libspectrum_id_t *type, const char *filename,
       { LIBSPECTRUM_ID_TAPE_LTP,      "ltp", 3, "\x11\0\0",	    0, 3, 1 },
       { LIBSPECTRUM_ID_TAPE_TZX,      "tzx", 3, "ZXTape!",	    0, 7, 4 },
       { LIBSPECTRUM_ID_TAPE_WARAJEVO, "tap", 2, "\xff\xff\xff\xff", 8, 4, 2 },
+      { LIBSPECTRUM_ID_TAPE_PZX,      "pzx", 3, "PZXT",		    0, 4, 4 },
 
       { LIBSPECTRUM_ID_DISK_SCL,      "scl", 3, "SINCLAIR",         0, 8, 4 },
       { LIBSPECTRUM_ID_DISK_TRD,      "trd", 3, NULL,		    0, 0, 0 },
@@ -552,6 +566,8 @@ libspectrum_identify_file_raw( libspectrum_id_t *type, const char *filename,
       { LIBSPECTRUM_ID_DISK_OPD,      "opd", 3, NULL,		    0, 0, 0 },
       { LIBSPECTRUM_ID_DISK_OPD,      "opu", 3, NULL,		    0, 0, 0 },
 
+      { LIBSPECTRUM_ID_AUX_POK,       "pok", 3, NULL,		    0, 0, 0 },
+
       { -1, NULL, 0, NULL, 0, 0, 0 }, /* End marker */
 
     };
@@ -708,6 +724,7 @@ libspectrum_identify_class( libspectrum_class_t *libspectrum_class,
   case LIBSPECTRUM_ID_TAPE_Z80EM:
   case LIBSPECTRUM_ID_TAPE_CSW:
   case LIBSPECTRUM_ID_TAPE_WAV:
+  case LIBSPECTRUM_ID_TAPE_PZX:
     *libspectrum_class = LIBSPECTRUM_CLASS_TAPE; return 0;
 
   case LIBSPECTRUM_ID_CARTRIDGE_IF2:
@@ -719,6 +736,8 @@ libspectrum_identify_class( libspectrum_class_t *libspectrum_class,
   case LIBSPECTRUM_ID_DISK_TD0:
     *libspectrum_class = LIBSPECTRUM_CLASS_DISK_GENERIC; return 0;
 
+  case LIBSPECTRUM_ID_AUX_POK:
+    *libspectrum_class = LIBSPECTRUM_CLASS_AUXILIARY; return 0;
   }
 
   libspectrum_print_error( LIBSPECTRUM_ERROR_UNKNOWN,
@@ -908,9 +927,7 @@ void
 libspectrum_make_room( libspectrum_byte **dest, size_t requested,
 		       libspectrum_byte **ptr, size_t *allocated )
 {
-  size_t current_length;
-
-  current_length = *ptr - *dest;
+  size_t current_length = 0;
 
   if( *allocated == 0 ) {
 
@@ -918,6 +935,7 @@ libspectrum_make_room( libspectrum_byte **dest, size_t requested,
     *dest = libspectrum_malloc( requested * sizeof( **dest ) );
 
   } else {
+    current_length = *ptr - *dest;
 
     /* If there's already enough room here, just return */
     if( current_length + requested <= (*allocated) ) return;
