@@ -1,7 +1,7 @@
 /* make-perl.c: Generate a perl script to create the libspectrum_* typedefs
    Copyright (c) 2002-2003 Philip Kendall, Darren Salt
 
-   $Id: make-perl.c 4123 2010-02-22 11:56:10Z fredm $
+   $Id: make-perl.c 4796 2012-12-26 10:36:57Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -133,6 +133,7 @@ int main(void)
 "typedef long glong;\n"
 "typedef gint gboolean;\n"
 "typedef unsigned int guint;\n"
+"typedef unsigned long gulong;\n"
 "typedef const void * gconstpointer;\n"
 "typedef void * gpointer;\n"
 "\n"
@@ -149,6 +150,10 @@ int main(void)
 "typedef gint		(*GCompareFunc)		(gconstpointer	a,\n"
 "						 gconstpointer	b);\n"
 "\n"
+"typedef void           (*GDestroyNotify)       (gpointer       data);\n"
+"\n"
+"typedef void           (*GFreeFunc)            (gpointer       data);\n"
+"\n"
 "\n"
 "WIN32_DLL GSList *g_slist_insert_sorted	(GSList		*list,\n"
 "						 gpointer	 data,\n"
@@ -165,7 +170,7 @@ int main(void)
 "						 gpointer	 data);\n"
 "\n"
 "WIN32_DLL GSList *g_slist_remove		(GSList		*list,\n"
-"						 gpointer	 data);\n"
+"						 gconstpointer	 data);\n"
 "\n"
 "WIN32_DLL GSList *g_slist_last			(GSList		*list);\n"
 "\n"
@@ -186,7 +191,7 @@ int main(void)
 "					 guint		n);\n"
 "\n"
 "WIN32_DLL GSList *g_slist_find_custom	(GSList		*list,\n"
-"					 gpointer	data,\n"
+"					 gconstpointer	data,\n"
 "					 GCompareFunc	func );\n"
 "\n"
 "WIN32_DLL gint	g_slist_position	(GSList		*list,\n"
@@ -196,6 +201,10 @@ int main(void)
 "\n"
 "typedef guint		(*GHashFunc)		(gconstpointer	key);\n"
 "\n"
+"typedef void	(*GHFunc)		(gpointer	key,\n"
+"						 gpointer	value,\n"
+"						 gpointer	user_data);\n"
+"\n"
 "typedef gboolean	(*GHRFunc)		(gpointer	key,\n"
 "						 gpointer	value,\n"
 "						 gpointer	user_data);\n"
@@ -211,6 +220,11 @@ int main(void)
 "WIN32_DLL GHashTable *g_hash_table_new	(GHashFunc	 hash_func,\n"
 "					 GCompareFunc	 key_compare_func);\n"
 "\n"
+"WIN32_DLL GHashTable *g_hash_table_new_full (GHashFunc       hash_func,\n"
+"                                             GCompareFunc    key_equal_func,\n"
+"                                             GDestroyNotify  key_destroy_func,\n"
+"                                             GDestroyNotify  value_destroy_func);\n"
+"\n"
 "WIN32_DLL void	g_hash_table_destroy	(GHashTable	*hash_table);\n"
 "\n"
 "WIN32_DLL void	g_hash_table_insert	(GHashTable	*hash_table,\n"
@@ -220,6 +234,10 @@ int main(void)
 "WIN32_DLL gpointer g_hash_table_lookup	(GHashTable	*hash_table,\n"
 "					 gconstpointer	 key);\n"
 "\n"
+"WIN32_DLL void	g_hash_table_foreach (GHashTable	*hash_table,\n"
+"						 GHFunc    func,\n"
+"						 gpointer  user_data);\n"
+"\n"
 "WIN32_DLL guint	g_hash_table_foreach_remove	(GHashTable	*hash_table,\n"
 "						 GHRFunc	 func,\n"
 "						 gpointer	 user_data);\n"
@@ -244,13 +262,16 @@ int main(void)
 "WIN32_DLL GArray* g_array_append_vals( GArray *array, gconstpointer data, guint len );\n"
 "#define g_array_index(a,t,i) (*(((t*)a->data)+i))\n"
 "WIN32_DLL GArray* g_array_set_size( GArray *array, guint length );\n"
+"WIN32_DLL gchar* g_array_free( GArray *array, gboolean free_segment );\n"
 "\n" );
   if( sizeof( void* ) == sizeof( int ) ) {
     printf( "#define GINT_TO_POINTER(i)      ((gpointer)  (i))\n" );
     printf( "#define GPOINTER_TO_INT(p)      ((gint)   (p))\n" );
+    printf( "#define GPOINTER_TO_UINT(p)     ((guint)  (p))\n" );
   } else if( sizeof( void* ) == sizeof( long ) ) {
     printf( "#define GINT_TO_POINTER(i)      ((gpointer)  (glong)(i))\n" );
     printf( "#define GPOINTER_TO_INT(p)      ((gint)   (glong)(p))\n" );
+    printf( "#define GPOINTER_TO_UINT(p)     ((guint)  (gulong)(p))\n" );
   } else {
     fprintf( stderr, "No plausible int to pointer cast found\n" );
     return 1;
@@ -309,6 +330,16 @@ int main(void)
 
 #endif				/* #ifdef HAVE_ZLIB_H */
 
+#ifdef HAVE_LIBBZ2
+  printf( "\n/* we support files compressed with bz2 */\n" );
+  printf( "#define	LIBSPECTRUM_SUPPORTS_BZ2_COMPRESSION	(1)\n\n" );
+#endif				/* #ifdef HAVE_LIBBZ2 */
+
+#ifdef HAVE_LIB_AUDIOFILE
+  printf( "\n/* we support wav files */\n" );
+  printf( "#define	LIBSPECTRUM_SUPPORTS_AUDIOFILE	(1)\n\n" );
+#endif				/* #ifdef HAVE_LIB_AUDIOFILE */
+
   printf( "CODE\n}\n\n" );
 
   return 0;
