@@ -1,7 +1,7 @@
 /* gslist.c: Minimal replacement for GSList
    Copyright (c) 2001-2004 Matan Ziv-Av, Philip Kendall, Marek Januszewski
 
-   $Id: gslist.c 4039 2009-06-27 06:04:44Z fredm $
+   $Id: gslist.c 4695 2012-05-07 02:03:10Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,12 +43,14 @@ gint	last_function		(gconstpointer	 a,
 static int FREE_LIST_ALLOCATE_CHUNK = 1024;
 
 GSList * free_list = NULL;
+GSList * allocated_list = NULL;
 
 static
 void    allocate_free   ( void ) {
     if(!free_list) {
         int i;
         free_list=libspectrum_malloc(FREE_LIST_ALLOCATE_CHUNK*sizeof(GSList));
+        allocated_list = free_list;
         for(i=0;i<FREE_LIST_ALLOCATE_CHUNK-1;i++)
             free_list[i].next=&free_list[i+1];
         free_list[FREE_LIST_ALLOCATE_CHUNK-1].next=NULL;
@@ -186,7 +188,7 @@ gint	last_function		(gconstpointer	 a,
 }
 
 GSList* g_slist_remove		(GSList		*list,
-				 gpointer	 data) {
+				 gconstpointer	 data) {
 
   GSList *tmp;
   GSList *prev;
@@ -326,7 +328,7 @@ GSList* g_slist_nth		(GSList		*list,
 }
 
 GSList* g_slist_find_custom	(GSList		*list,
-				 gpointer	data,
+				 gconstpointer	data,
 				 GCompareFunc	func ) {
   while (list)
     {
@@ -352,4 +354,12 @@ gint	g_slist_position	(GSList		*list,
   return -1;
 }
 
+void
+libspectrum_slist_cleanup( void )
+{
+  libspectrum_free( allocated_list );
+  allocated_list = NULL;
+  free_list = NULL;
+}
+
 #endif				/* #ifndef HAVE_LIB_GLIB */
