@@ -1,7 +1,7 @@
 /* rzx.c: routines for dealing with .rzx files
    Copyright (c) 2002-2008 Philip Kendall
 
-   $Id: rzx.c 4157 2010-09-12 02:11:22Z fredm $
+   $Id: rzx.c 4780 2012-11-29 19:09:52Z sbaldovi $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -194,7 +194,9 @@ block_free( rzx_block_t *block )
 {
   size_t i;
   input_block_t *input;
+#ifdef HAVE_GCRYPT_H
   signature_block_t *signature;
+#endif				/* #ifdef HAVE_GCRYPT_H */
 
   switch( block->type ) {
 
@@ -216,9 +218,8 @@ block_free( rzx_block_t *block )
     return LIBSPECTRUM_ERROR_NONE;
 
   case LIBSPECTRUM_RZX_SIGN_END_BLOCK:
-    signature = &( block->types.signature );
-
 #ifdef HAVE_GCRYPT_H
+    signature = &( block->types.signature );
     gcry_mpi_release( signature->r );
     gcry_mpi_release( signature->s );
 #endif				/* #ifdef HAVE_GCRYPT_H */
@@ -766,8 +767,6 @@ libspectrum_rzx_read( libspectrum_rzx *rzx, const libspectrum_byte *buffer,
 static libspectrum_error
 rzx_read_header( const libspectrum_byte **ptr, const libspectrum_byte *end )
 {
-  libspectrum_dword flags;
-
   /* Check the header exists */
   if( end - (*ptr) < (ptrdiff_t)strlen( rzx_signature ) + 6 ) {
     libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
@@ -785,7 +784,9 @@ rzx_read_header( const libspectrum_byte **ptr, const libspectrum_byte *end )
   /* Skip over the signature and the version numbers */
   (*ptr) += strlen( rzx_signature ) + 2;
 
-  flags = libspectrum_read_dword( ptr );
+  /* And skip the flags as well - separate call as we'll need the return
+     value if we ever fix the stuff below */
+  libspectrum_read_dword( ptr );
 
   /* FIXME: how to handle signatures */
 
@@ -1448,7 +1449,6 @@ static libspectrum_error
 rzx_write_input( input_block_t *block, libspectrum_byte **buffer,
 		 libspectrum_byte **ptr, size_t *length, int compress )
 {
-  libspectrum_error error;
   size_t i, size;
   size_t length_offset, data_offset, flags_offset;
   libspectrum_byte *length_ptr; 
@@ -1512,6 +1512,7 @@ rzx_write_input( input_block_t *block, libspectrum_byte **buffer,
     /* Compress the data the simple way. Really, we should stream the data */
     libspectrum_byte *gzsnap = NULL; size_t gzlength;
     libspectrum_byte *data_ptr = *buffer + data_offset;
+    libspectrum_error error;
 
     error = libspectrum_zlib_compress( data_ptr, *ptr - data_ptr,
 				       &gzsnap, &gzlength );
