@@ -1,7 +1,7 @@
 /* sna.c: Routines for handling .sna snapshots
    Copyright (c) 2001-2002 Philip Kendall
 
-   $Id: sna.c 4148 2010-08-25 21:20:50Z pak21 $
+   $Id: sna.c 4836 2012-12-31 15:40:10Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -310,7 +310,7 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
   if( libspectrum_snap_divide_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
-  /* We don't save the Interface II ROM at all */
+  /* We don't save the Interface 2 ROM at all */
   if( libspectrum_snap_interface2_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
@@ -327,6 +327,14 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
       libspectrum_snap_melodik_active( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
+  /* We don't save the Specdrum state at all */
+  if( libspectrum_snap_specdrum_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  /* We don't save the Spectranet state at all */
+  if( libspectrum_snap_spectranet_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
   ptr = *buffer;
 
   write_header( buffer, &ptr, length, snap );
@@ -345,6 +353,7 @@ libspectrum_sna_write( libspectrum_byte **buffer, size_t *length,
     break;
     
   case LIBSPECTRUM_MACHINE_128:
+  case LIBSPECTRUM_MACHINE_128E:
   case LIBSPECTRUM_MACHINE_PENT512:
   case LIBSPECTRUM_MACHINE_PENT1024:
   case LIBSPECTRUM_MACHINE_PLUS2:
