@@ -1,7 +1,7 @@
 /* tap.c: Routines for handling .tap files
    Copyright (c) 2001-2003 Philip Kendall
 
-   $Id: tap.c 3784 2008-10-22 12:36:07Z fredm $
+   $Id: tap.c 4440 2011-05-15 10:53:47Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -126,7 +126,7 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
     ptr += buf_length;
 
     /* Give a 1s pause after each block */
-    libspectrum_tape_block_set_pause( block, 1000 );
+    libspectrum_set_pause_ms( block, 1000 );
 
     libspectrum_tape_append_block( tape, block );
   }
@@ -177,6 +177,8 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_LOOP_START:     /* Could do better? */
     case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
     case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
+    case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+    case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
       error = skip_block( block, "conversion almost certainly won't work" );
       if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
       done = 1;
@@ -185,6 +187,7 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_PAUSE:
     case LIBSPECTRUM_TAPE_BLOCK_JUMP:
     case LIBSPECTRUM_TAPE_BLOCK_SELECT:
+    case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
       error = skip_block( block, "conversion may not work" );
       if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
       done = 1;
