@@ -1,7 +1,7 @@
 /* tape.c: Routines for handling tape files
    Copyright (c) 2001-2008 Philip Kendall, Darren Salt, Fredrick Meunier
 
-   $Id: tape.c 4077 2009-09-01 13:05:45Z fredm $
+   $Id: tape.c 4436 2011-05-14 14:10:07Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_SYNC1 =  667; /*Sync 1*/
 const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_SYNC2 =  735; /*Sync 2*/
 const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_DATA0 =  855; /*Reset*/
 const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_DATA1 = 1710; /*Set*/
+const libspectrum_dword LIBSPECTRUM_TAPE_TIMING_TAIL  =  945; /*Tail*/
 
 /*** Local function prototypes ***/
 
@@ -97,7 +98,8 @@ pure_data_edge( libspectrum_tape_pure_data_block *block,
 static libspectrum_error
 raw_data_edge( libspectrum_tape_raw_data_block *block,
                libspectrum_tape_raw_data_block_state *state,
-	       libspectrum_dword *tstates, int *end_of_block );
+	       libspectrum_dword *tstates, int *end_of_block,
+               int *flags );
 
 static libspectrum_error
 jump_blocks( libspectrum_tape *tape, int offset );
@@ -107,6 +109,21 @@ rle_pulse_edge( libspectrum_tape_rle_pulse_block *block,
                 libspectrum_tape_rle_pulse_block_state *state,
 		libspectrum_dword *tstates, int *end_of_block );
 
+static libspectrum_error
+pulse_sequence_edge( libspectrum_tape_pulse_sequence_block *block,
+                     libspectrum_tape_pulse_sequence_block_state *state,
+                     libspectrum_dword *tstates, int *end_of_block,
+                     int *flags );
+
+libspectrum_error
+libspectrum_tape_data_block_next_bit( libspectrum_tape_data_block *block,
+                                    libspectrum_tape_data_block_state *state );
+
+static libspectrum_error
+data_block_edge( libspectrum_tape_data_block *block,
+                 libspectrum_tape_data_block_state *state,
+                 libspectrum_dword *tstates, int *end_of_block, int *flags );
+
 /*** Function definitions ****/
 
 /* Allocate a list of blocks */
@@ -231,6 +248,9 @@ libspectrum_tape_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
     break;
 #endif    /* #ifdef HAVE_LIB_AUDIOFILE */
 
+  case LIBSPECTRUM_ID_TAPE_PZX:
+    error = internal_pzx_read( tape, buffer, length ); break;
+
   default:
     libspectrum_print_error( LIBSPECTRUM_ERROR_CORRUPT,
 			     "libspectrum_tape_read: not a tape file" );
@@ -298,8 +318,10 @@ const int LIBSPECTRUM_TAPE_FLAGS_STOP48     = 1 << 2; /* Stop tape if in
 const int LIBSPECTRUM_TAPE_FLAGS_NO_EDGE    = 1 << 3; /* Not an edge really */
 const int LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW  = 1 << 4; /* Set level low */
 const int LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH = 1 << 5; /* Set level high */
-const int LIBSPECTRUM_TAPE_FLAGS_LENGTH_SHORT = 1 << 6;
-const int LIBSPECTRUM_TAPE_FLAGS_LENGTH_LONG = 1 << 7;
+const int LIBSPECTRUM_TAPE_FLAGS_LENGTH_SHORT = 1 << 6;/* Short edge; used for
+                                                          loader acceleration */
+const int LIBSPECTRUM_TAPE_FLAGS_LENGTH_LONG = 1 << 7; /* Long edge; used for
+                                                          loader acceleration */
 const int LIBSPECTRUM_TAPE_FLAGS_TAPE       = 1 << 8; /* End of tape */
 
 libspectrum_error
@@ -352,7 +374,7 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
       break;
     case LIBSPECTRUM_TAPE_BLOCK_RAW_DATA:
       error = raw_data_edge( &(block->types.raw_data), &(it->block_state.raw_data),
-                             tstates, &end_of_block );
+                             tstates, &end_of_block, flags );
       if( error ) return error;
       break;
 
@@ -364,7 +386,14 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_PAUSE:
-      *tstates = ( block->types.pause.length * 69888 ) / 20; end_of_block = 1;
+      *tstates = block->types.pause.length_tstates; end_of_block = 1;
+      /* If the pause isn't a "don't care" level then set the appropriate pulse
+         level */
+      if( block->types.pause.level != -1 &&
+          block->types.pause.length_tstates ) {
+        *flags |= block->types.pause.level ? LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH :
+                                             LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW;
+      }
       /* 0 ms pause => stop tape */
       if( *tstates == 0 ) { *flags |= LIBSPECTRUM_TAPE_FLAGS_STOP; }
       break;
@@ -372,7 +401,8 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
     case LIBSPECTRUM_TAPE_BLOCK_JUMP:
       error = jump_blocks( tape, block->types.jump.offset );
       if( error ) return error;
-      *tstates = 0; end_of_block = 1; no_advance = 1;
+      *tstates = 0; *flags |= LIBSPECTRUM_TAPE_FLAGS_NO_EDGE; end_of_block = 1;
+      no_advance = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_LOOP_START:
@@ -380,7 +410,7 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
         it->loop_block = it->current_block->next;
         it->loop_count = block->types.loop_start.count;
       }
-      *tstates = 0; end_of_block = 1;
+      *tstates = 0; *flags |= LIBSPECTRUM_TAPE_FLAGS_NO_EDGE; end_of_block = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
@@ -392,13 +422,21 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
           it->loop_block = NULL;
         }
       }
-      *tstates = 0; end_of_block = 1;
+      *tstates = 0; *flags |= LIBSPECTRUM_TAPE_FLAGS_NO_EDGE; end_of_block = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_STOP48:
       *tstates = 0; *flags |= LIBSPECTRUM_TAPE_FLAGS_STOP48; end_of_block = 1;
       break;
 
+    case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
+      *tstates = 0; end_of_block = 1;
+      /* Inverted as the following block will flip the level before recording
+         the edge */
+      *flags |= block->types.set_signal_level.level ?
+          LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW : LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH;
+      break;
+
     /* For blocks which contain no Spectrum-readable data, return zero
        tstates and set end of block set so we instantly get the next block */
     case LIBSPECTRUM_TAPE_BLOCK_GROUP_START: 
@@ -409,7 +447,7 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
     case LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO:
     case LIBSPECTRUM_TAPE_BLOCK_HARDWARE:
     case LIBSPECTRUM_TAPE_BLOCK_CUSTOM:
-      *tstates = 0; end_of_block = 1;
+      *tstates = 0; *flags |= LIBSPECTRUM_TAPE_FLAGS_NO_EDGE; end_of_block = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
@@ -418,6 +456,20 @@ libspectrum_tape_get_next_edge_internal( libspectrum_dword *tstates,
       if( error ) return error;
       break;
 
+    case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+      error = pulse_sequence_edge( &(block->types.pulse_sequence),
+                                   &(it->block_state.pulse_sequence), tstates,
+                                   &end_of_block, flags );
+      if( error ) return error;
+      break;
+
+    case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
+      error = data_block_edge( &(block->types.data_block),
+                               &(it->block_state.data_block), tstates,
+                               &end_of_block, flags );
+      if( error ) return error;
+      break;
+
     default:
       *tstates = 0;
       libspectrum_print_error(
@@ -523,8 +575,7 @@ rom_edge( libspectrum_tape_rom_block *block,
 
   case LIBSPECTRUM_TAPE_STATE_PAUSE:
     /* The pause at the end of the block */
-    *tstates = (block->pause * 69888)/20; /* FIXME: should vary with tstates
-					     per frame */
+    *tstates = block->pause_tstates;
     *end_of_block = 1;
     break;
 
@@ -621,8 +672,7 @@ turbo_edge( libspectrum_tape_turbo_block *block,
 
   case LIBSPECTRUM_TAPE_STATE_PAUSE:
     /* The pause at the end of the block */
-    *tstates = (block->pause * 69888)/20; /* FIXME: should vary with tstates
-					     per frame */
+    *tstates = block->pause_tstates;
     *end_of_block = 1;
     break;
 
@@ -730,8 +780,7 @@ pure_data_edge( libspectrum_tape_pure_data_block *block,
 
   case LIBSPECTRUM_TAPE_STATE_PAUSE:
     /* The pause at the end of the block */
-    *tstates = (block->pause * 69888)/20; /* FIXME: should vary with tstates
-					     per frame */
+    *tstates = block->pause_tstates;
     *end_of_block = 1;
     break;
 
@@ -789,18 +838,22 @@ libspectrum_tape_pure_data_next_bit( libspectrum_tape_pure_data_block *block,
 static libspectrum_error
 raw_data_edge( libspectrum_tape_raw_data_block *block,
                libspectrum_tape_raw_data_block_state *state,
-	       libspectrum_dword *tstates, int *end_of_block )
+	       libspectrum_dword *tstates, int *end_of_block,
+               int *flags )
 {
   switch (state->state) {
   case LIBSPECTRUM_TAPE_STATE_DATA1:
     *tstates = state->bit_tstates;
     libspectrum_tape_raw_data_next_bit( block, state );
+    /* Backwards as last bit is the state of the next bit as it has already been
+       updated*/
+    *flags |= state->last_bit ? LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW :
+                                LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH;
     break;
 
   case LIBSPECTRUM_TAPE_STATE_PAUSE:
     /* The pause at the end of the block */
-    *tstates = ( block->pause * 69888 )/20; /* FIXME: should vary with tstates
-					       per frame */
+    *tstates = block->pause_tstates;
     *end_of_block = 1;
     break;
 
@@ -960,8 +1013,7 @@ generalised_data_edge( libspectrum_tape_generalised_data_block *block,
 
   case LIBSPECTRUM_TAPE_STATE_PAUSE:
     /* The pause at the end of the block */
-    *tstates = ( block->pause * 69888 )/20; /* FIXME: should vary with tstates
-					       per frame */
+    *tstates = block->pause_tstates;
     *end_of_block = 1;
     break;
 
@@ -1023,6 +1075,131 @@ rle_pulse_edge( libspectrum_tape_rle_pulse_block *block,
   return LIBSPECTRUM_ERROR_NONE;
 }
 
+static libspectrum_error
+pulse_sequence_edge( libspectrum_tape_pulse_sequence_block *block,
+                     libspectrum_tape_pulse_sequence_block_state *state,
+                     libspectrum_dword *tstates, int *end_of_block, int *flags )
+{
+  int new_level = state->level;
+  /* Get the length of this edge */
+  *tstates = 0;
+  /* Skip past any 0 blocks until we find a non 0 block or reach the end of the
+     block, keeping track of the current mic level */
+  while( !( *tstates || *end_of_block ) ) {
+    *tstates = block->lengths[ state->index ];
+    new_level = !new_level;
+    /* Was that the last repeat of this pulse block? */
+    if( ++(state->pulse_count) == block->pulse_repeats[ state->index ] ) {
+      state->index++;
+      /* Was that the last block available? */
+      if( state->index >= block->count ) {
+        /* Next block */
+        (*end_of_block) = 1;
+      } else {
+        /* Next pulse block */
+        state->pulse_count = 0;
+      }
+    }
+  }
+
+  if( new_level != state->level ) {
+    *flags |= new_level ? LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH :
+                          LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW;
+    state->level = new_level;
+  } else if( !(*tstates) ) {
+    /* If the net effect of this edge was 0 tstates and no level change, it was
+       much ado about nothing */
+    *flags |= LIBSPECTRUM_TAPE_FLAGS_NO_EDGE;
+  }
+
+  return LIBSPECTRUM_ERROR_NONE;
+}
+
+libspectrum_error
+libspectrum_tape_data_block_next_bit( libspectrum_tape_data_block *block,
+                                      libspectrum_tape_data_block_state *state )
+{
+  int next_bit;
+
+  /* Have we finished the current byte? */
+  if( ++(state->bits_through_byte) == 8 ) {
+
+    /* If so, have we finished the entire block? If so, all we've got
+       left after this is the tail at the end */
+    if( ++(state->bytes_through_block) == block->length ) {
+      state->state = LIBSPECTRUM_TAPE_STATE_TAIL;
+      return LIBSPECTRUM_ERROR_NONE;
+    }
+    
+    /* If we've finished the current byte, but not the entire block,
+       get the next byte */
+    state->current_byte = block->data[ state->bytes_through_block ];
+
+    /* If we're looking at the last byte, take account of the fact it
+       may have less than 8 bits in it */
+    if( state->bytes_through_block == block->length-1 ) {
+      state->bits_through_byte = 8 - block->bits_in_last_byte;
+    } else {
+      state->bits_through_byte = 0;
+    }
+  }
+
+  /* Get the high bit, and shift the byte out leftwards */
+  next_bit = state->current_byte & 0x80;
+  state->current_byte <<= 1;
+
+  /* And set state for another data bit */
+  state->bit_pulses = ( next_bit ? block->bit1_pulses : block->bit0_pulses );
+  state->pulse_count = ( next_bit ? block->bit1_pulse_count :
+                                    block->bit0_pulse_count );
+  state->bit_flags = ( next_bit ? state->bit1_flags : state->bit0_flags );
+  state->index = 0;
+  state->state = LIBSPECTRUM_TAPE_STATE_DATA1;
+
+  return LIBSPECTRUM_ERROR_NONE;
+}
+
+static libspectrum_error
+data_block_edge( libspectrum_tape_data_block *block,
+                 libspectrum_tape_data_block_state *state,
+		 libspectrum_dword *tstates, int *end_of_block, int *flags )
+{
+  int error;
+
+  switch( state->state ) {
+
+  case LIBSPECTRUM_TAPE_STATE_DATA1:
+    /* The next pulse for a bit of data */
+    *tstates = state->bit_pulses[ state->index ];
+    *flags |= state->bit_flags;
+    if( ++(state->index) == state->pulse_count ) {
+      /* Followed by the next bit of data (or the end of data) */
+      error = libspectrum_tape_data_block_next_bit( block, state );
+      if( error ) return error;
+    }
+    break;
+
+  case LIBSPECTRUM_TAPE_STATE_TAIL:
+    /* The pulse at the end of the block */
+    *tstates = block->tail_length;
+    *end_of_block = 1;
+    break;
+
+  default:
+    libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
+			     "data_block_edge: unknown state %d",
+			     state->state );
+    return LIBSPECTRUM_ERROR_LOGIC;
+
+  }
+
+  *flags |= state->level ? LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH :
+                           LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW;
+  state->level = !state->level;
+
+  return LIBSPECTRUM_ERROR_NONE;
+}
+
 /* Get the current block */
 libspectrum_tape_block*
 libspectrum_tape_current_block( libspectrum_tape *tape )
@@ -1207,6 +1384,10 @@ libspectrum_tape_block_description( char *buffer, size_t length,
     strncpy( buffer, "Stop Tape If In 48K Mode", length );
     break;
 
+  case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
+    strncpy( buffer, "Set Signal Level", length );
+    break;
+
   case LIBSPECTRUM_TAPE_BLOCK_COMMENT:
     strncpy( buffer, "Comment", length );
     break;
@@ -1228,6 +1409,14 @@ libspectrum_tape_block_description( char *buffer, size_t length,
     strncpy( buffer, "RLE Pulse", length );
     break;
 
+  case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+    strncpy( buffer, "Pulse Sequence", length );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
+    strncpy( buffer, "Data Block", length );
+    break;
+
   default:
     libspectrum_print_error(
       LIBSPECTRUM_ERROR_LOGIC,
