@@ -1,7 +1,7 @@
 /* tape_block.c: one tape block
    Copyright (c) 2003-2008 Philip Kendall
 
-   $Id: tape_block.c 4136 2010-05-26 12:41:32Z fredm $
+   $Id: tape_block.c 4433 2011-05-14 05:44:47Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 
 #include <config.h>
 
+#include <math.h>
+
 #include "tape_block.h"
 
 /* The number of pilot pulses for the standard ROM loader NB: These
@@ -37,8 +39,6 @@
 static const size_t LIBSPECTRUM_TAPE_PILOTS_HEADER = 0x1f7f;
 static const size_t LIBSPECTRUM_TAPE_PILOTS_DATA   = 0x0c97;
 
-#define TZX_HZ 3500000
-
 /* Functions to initialise block types */
 
 static libspectrum_error
@@ -56,6 +56,9 @@ raw_data_init( libspectrum_tape_raw_data_block *block,
 static libspectrum_error
 generalised_data_init( libspectrum_tape_generalised_data_block *block,
                        libspectrum_tape_generalised_data_block_state *state );
+static libspectrum_error
+data_block_init( libspectrum_tape_data_block *block,
+                 libspectrum_tape_data_block_state *state );
 
 libspectrum_tape_block*
 libspectrum_tape_block_alloc( libspectrum_tape_type type )
@@ -136,6 +139,9 @@ libspectrum_tape_block_free( libspectrum_tape_block *block )
   case LIBSPECTRUM_TAPE_BLOCK_STOP48:
     break;
 
+  case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
+    break;
+
   case LIBSPECTRUM_TAPE_BLOCK_COMMENT:
     libspectrum_free( block->types.comment.text );
     break;
@@ -160,12 +166,23 @@ libspectrum_tape_block_free( libspectrum_tape_block *block )
     libspectrum_free( block->types.custom.data );
     break;
 
-    /* Block types not present in .tzx follow here */
+  /* Block types not present in .tzx follow here */
 
   case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
     libspectrum_free( block->types.rle_pulse.data );
     break;
 
+  case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+    libspectrum_free( block->types.pulse_sequence.lengths );
+    libspectrum_free( block->types.pulse_sequence.pulse_repeats );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
+    libspectrum_free( block->types.data_block.data );
+    libspectrum_free( block->types.data_block.bit0_pulses );
+    libspectrum_free( block->types.data_block.bit1_pulses );
+    break;
+
   case LIBSPECTRUM_TAPE_BLOCK_CONCAT: /* This should never occur */
   default:
     libspectrum_print_error( LIBSPECTRUM_ERROR_LOGIC,
@@ -224,6 +241,14 @@ libspectrum_tape_block_init( libspectrum_tape_block *block,
   case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
     state->block_state.rle_pulse.index = 0;
     return LIBSPECTRUM_ERROR_NONE;
+  case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+    state->block_state.pulse_sequence.index = 0;
+    state->block_state.pulse_sequence.pulse_count = 0;
+    state->block_state.pulse_sequence.level = 1;
+    return LIBSPECTRUM_ERROR_NONE;
+  case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
+    return data_block_init( &(block->types.data_block),
+                            &(state->block_state.data_block) );
 
   /* These blocks need no initialisation */
   case LIBSPECTRUM_TAPE_BLOCK_PAUSE:
@@ -234,6 +259,7 @@ libspectrum_tape_block_init( libspectrum_tape_block *block,
   case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
   case LIBSPECTRUM_TAPE_BLOCK_SELECT:
   case LIBSPECTRUM_TAPE_BLOCK_STOP48:
+  case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
   case LIBSPECTRUM_TAPE_BLOCK_COMMENT:
   case LIBSPECTRUM_TAPE_BLOCK_MESSAGE:
   case LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO:
@@ -331,6 +357,41 @@ generalised_data_init( libspectrum_tape_generalised_data_block *block,
   return LIBSPECTRUM_ERROR_NONE;
 }
 
+static libspectrum_error
+data_block_init( libspectrum_tape_data_block *block,
+                 libspectrum_tape_data_block_state *state )
+{
+  libspectrum_error error;
+
+  state->bit0_flags = 0;
+  state->bit1_flags = 0;
+
+  /* If both bit0 and bit1 are two matching pulses, then assign them to be
+     flagged as short/long pulses respectively */
+  if( block->bit0_pulse_count == 2 && block->bit1_pulse_count == 2 &&
+      block->bit0_pulses[ 0 ] == block->bit0_pulses[ 1 ] &&
+      block->bit1_pulses[ 0 ] == block->bit1_pulses[ 1 ] &&
+      block->bit0_pulses[ 0 ] > 0  && block->bit1_pulses[ 0 ] > 0 ) {
+    if( block->bit0_pulses[ 0 ] < block->bit1_pulses[ 0 ] ) {
+      state->bit0_flags = LIBSPECTRUM_TAPE_FLAGS_LENGTH_SHORT;
+      state->bit1_flags = LIBSPECTRUM_TAPE_FLAGS_LENGTH_LONG;
+    } else if( block->bit0_pulses[ 0 ] > block->bit1_pulses[ 0 ] ) {
+      state->bit0_flags = LIBSPECTRUM_TAPE_FLAGS_LENGTH_LONG;
+      state->bit1_flags = LIBSPECTRUM_TAPE_FLAGS_LENGTH_SHORT;
+    }
+  }
+
+  state->level = block->initial_level;
+
+  /* We're just before the start of the data */
+  state->bytes_through_block = -1; state->bits_through_byte = 7;
+  /* Set up the next bit */
+  error = libspectrum_tape_data_block_next_bit( block, state );
+  if( error ) return error;
+
+  return LIBSPECTRUM_ERROR_NONE;
+}
+
 /* Does this block consist solely of metadata? */
 int
 libspectrum_tape_block_metadata( libspectrum_tape_block *block )
@@ -348,7 +409,10 @@ libspectrum_tape_block_metadata( libspectrum_tape_block *block )
   case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
   case LIBSPECTRUM_TAPE_BLOCK_SELECT:
   case LIBSPECTRUM_TAPE_BLOCK_STOP48:
+  case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
   case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
+  case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+  case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
     return 0;
 
   case LIBSPECTRUM_TAPE_BLOCK_GROUP_START:
@@ -445,7 +509,7 @@ libspectrum_tape_block_zero( libspectrum_tape_block *block )
 #define BITS_SET_ARRAY_SIZE 256
 static int bits_set[ BITS_SET_ARRAY_SIZE ];
 
-static const int LIBSPECTRUM_BITS_IN_BYTE = 8;
+const int LIBSPECTRUM_BITS_IN_BYTE = 8;
 
 static int
 libspectrum_bits_set_n_bits( libspectrum_byte byte, libspectrum_byte bits )
@@ -485,19 +549,19 @@ convert_pulses_to_tstates( libspectrum_dword set_bit_length,
          ( pulses_in_bit * unset_bit_length ) * ( bit_block_size - bits_set_in_block );
 }
 
-static libspectrum_dword
-convert_ms_to_tstates( libspectrum_dword ms )
-{
-  static const libspectrum_dword tstates_per_ms = TZX_HZ / 1000;
-  return ms * tstates_per_ms;
-}
-
 static libspectrum_dword
 rom_block_length( libspectrum_tape_rom_block *rom )
 {
-  libspectrum_dword length = convert_ms_to_tstates( rom->pause );
+  libspectrum_dword length = rom->pause_tstates;
   size_t i;
 
+  size_t edge_count = rom->length && rom->data[0] & 0x80 ?
+                      LIBSPECTRUM_TAPE_PILOTS_DATA           :
+                      LIBSPECTRUM_TAPE_PILOTS_HEADER;
+  length += LIBSPECTRUM_TAPE_TIMING_PILOT * edge_count;
+  length += LIBSPECTRUM_TAPE_TIMING_SYNC1;
+  length += LIBSPECTRUM_TAPE_TIMING_SYNC2;
+
   for( i = 0; i < rom->length; i++ ) {
     libspectrum_byte data = rom->data[ i ];
     length += convert_pulses_to_tstates( LIBSPECTRUM_TAPE_TIMING_DATA1,
@@ -515,7 +579,7 @@ turbo_block_length( libspectrum_tape_turbo_block *turbo )
   libspectrum_dword length =
     turbo->pilot_pulses * turbo->pilot_length +
     turbo->sync1_length + turbo->sync2_length +
-    convert_ms_to_tstates( turbo->pause );
+    turbo->pause_tstates;
   size_t i;
   if( turbo->length ) {
     int bits_set_in_last_byte =
@@ -554,7 +618,7 @@ pulses_block_length( libspectrum_tape_pulses_block *pulses )
 static libspectrum_dword
 pure_data_block_length( libspectrum_tape_pure_data_block *pure_data )
 {
-  libspectrum_dword length = convert_ms_to_tstates( pure_data->pause );
+  libspectrum_dword length = pure_data->pause_tstates;
   size_t i;
   if( pure_data->length ) {
     int bits_set_in_last_byte =
@@ -582,7 +646,7 @@ pure_data_block_length( libspectrum_tape_pure_data_block *pure_data )
 static libspectrum_dword
 raw_data_block_length( libspectrum_tape_raw_data_block *raw_data )
 {
-  libspectrum_dword length = convert_ms_to_tstates( raw_data->pause );
+  libspectrum_dword length = raw_data->pause_tstates;
 
   length += ( LIBSPECTRUM_BITS_IN_BYTE * raw_data->length -
               ( LIBSPECTRUM_BITS_IN_BYTE - raw_data->bits_in_last_byte ) ) *
@@ -631,6 +695,61 @@ generalised_data_block_length(
   return length;
 }
 
+static libspectrum_dword
+pulse_sequence_block_length(
+                    libspectrum_tape_pulse_sequence_block *pulses )
+{
+  libspectrum_dword length = 0;
+  size_t i;
+
+  for( i = 0; i < pulses->count; i++ )
+    length += pulses->lengths[ i ] * pulses->pulse_repeats[ i ];
+
+  return length;
+}
+
+static libspectrum_dword
+data_block_length( libspectrum_tape_data_block *data_block )
+{
+  libspectrum_dword length = 0;
+  size_t i;
+  if( data_block->count ) {
+    int bits_set_in_last_byte =
+      libspectrum_bits_set_n_bits( data_block->data[ data_block->length-1 ],
+                                   data_block->bits_in_last_byte );
+    size_t bit0_length = 0;
+    size_t bit1_length = 0;
+
+    for( i = 0; i < data_block->bit0_pulse_count; i++ ) {
+      bit0_length += data_block->bit0_pulses[ i ];
+    }
+    bit0_length /=
+      data_block->bit0_pulse_count ? data_block->bit0_pulse_count : 1;
+
+    for( i = 0; i < data_block->bit1_pulse_count; i++ ) {
+      bit1_length += data_block->bit1_pulses[ i ];
+    }
+    bit1_length /=
+      data_block->bit1_pulse_count ? data_block->bit1_pulse_count : 1;
+
+    for( i = 0; i < data_block->length-1; i++ ) {
+      libspectrum_byte data = data_block->data[ i ];
+      length += convert_pulses_to_tstates( bit1_length,
+                                           bit0_length,
+                                           bits_set[ data ],
+                                           LIBSPECTRUM_BITS_IN_BYTE );
+    }
+      
+    /* handle bits in last byte correctly */
+    length += convert_pulses_to_tstates( bit1_length,
+                                         bit0_length,
+                                         bits_set_in_last_byte,
+                                         data_block->bits_in_last_byte );
+  }
+
+  return length;
+}
+
 libspectrum_dword
 libspectrum_tape_block_length( libspectrum_tape_block *block )
 {
@@ -649,19 +768,24 @@ libspectrum_tape_block_length( libspectrum_tape_block *block )
   case LIBSPECTRUM_TAPE_BLOCK_PURE_DATA:
     return pure_data_block_length( &block->types.pure_data );
   case LIBSPECTRUM_TAPE_BLOCK_PAUSE:
-    return convert_ms_to_tstates( block->types.pause.length );
+    return block->types.pause.length_tstates;
   case LIBSPECTRUM_TAPE_BLOCK_RAW_DATA:
     return raw_data_block_length( &block->types.raw_data );
   case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
     return rle_pulse_block_length( &block->types.rle_pulse );
   case LIBSPECTRUM_TAPE_BLOCK_GENERALISED_DATA:
     return generalised_data_block_length( &block->types.generalised_data );
+  case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
+    return pulse_sequence_block_length( &block->types.pulse_sequence );
+  case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
+    return data_block_length( &block->types.data_block );
   case LIBSPECTRUM_TAPE_BLOCK_GROUP_START:
   case LIBSPECTRUM_TAPE_BLOCK_GROUP_END:
   case LIBSPECTRUM_TAPE_BLOCK_JUMP:
   case LIBSPECTRUM_TAPE_BLOCK_LOOP_START:
   case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
   case LIBSPECTRUM_TAPE_BLOCK_STOP48:
+  case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
   case LIBSPECTRUM_TAPE_BLOCK_COMMENT:
   case LIBSPECTRUM_TAPE_BLOCK_MESSAGE:
   case LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO:
