@@ -1,7 +1,8 @@
 #include "test.h"
 
 test_return_t
-check_edges( const char *filename, test_edge_sequence_t *edges )
+check_edges( const char *filename, test_edge_sequence_t *edges,
+	     int flags_mask )
 {
   libspectrum_byte *buffer = NULL;
   size_t filesize = 0;
@@ -11,19 +12,16 @@ check_edges( const char *filename, test_edge_sequence_t *edges )
 
   if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
 
-  if( libspectrum_tape_alloc( &tape ) ) {
-    free( buffer );
-    return TEST_INCOMPLETE;
-  }
+  tape = libspectrum_tape_alloc();
 
   if( libspectrum_tape_read( tape, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
 			     filename ) != LIBSPECTRUM_ERROR_NONE ) {
     libspectrum_tape_free( tape );
-    free( buffer );
+    libspectrum_free( buffer );
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   while( 1 ) {
 
@@ -37,6 +35,8 @@ check_edges( const char *filename, test_edge_sequence_t *edges )
       return TEST_INCOMPLETE;
     }
 
+    flags &= flags_mask;
+
     if( tstates != ptr->length || flags != ptr->flags ) {
       fprintf( stderr, "%s: expected %d tstates and flags %d, got %d tstates and flags %d\n",
 	       progname, ptr->length, ptr->flags, tstates, flags );
