@@ -1,7 +1,7 @@
 /* z80.c: Routines for handling .z80 snapshots
    Copyright (c) 2001-2009 Philip Kendall, Darren Salt, Fredrick Meunier
 
-   $Id: z80.c 4163 2010-09-29 21:29:00Z pak21 $
+   $Id: z80.c 4836 2012-12-31 15:40:10Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -93,7 +93,7 @@ enum {
   Z80_MACHINE_FIRST_EXTENSION = Z80_MACHINE_PLUS3,
 };
 
-/* Constants representing a Sinclair Interface II port 1 joystick as a .z80
+/* Constants representing a Sinclair Interface 2 port 1 joystick as a .z80
    user defined joystick */
 static const libspectrum_word if2_left_map_l = 0x0f03;
 static const libspectrum_word if2_left_map_r = 0x0803;
@@ -1125,11 +1125,18 @@ libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
 
   *out_flags = 0;
 
-  /* .z80 format doesn't store the 'last instruction EI' or 'halted' state */
+  /* .z80 format doesn't store the 'last instruction EI', 'halted' state  or
+     the 'last instruction set Flags' */
   if( libspectrum_snap_last_instruction_ei( snap ) ) 
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
   if( libspectrum_snap_halted( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
+  if( libspectrum_snap_last_instruction_set_f( snap ) ) 
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
+
+  /* .z80 format doesn't store the 'late timings' state */
+  if( libspectrum_snap_late_timings( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MINOR_INFO_LOSS;
 
   /* .z80 format doesn't store +D info well */
   if( libspectrum_snap_plusd_active( snap ) )
@@ -1167,6 +1174,14 @@ libspectrum_z80_write2( libspectrum_byte **buffer, size_t *length,
   if( libspectrum_snap_custom_rom( snap ) )
     *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
 
+  /* .z80 format doesn't save the Specdrum state at all */
+  if( libspectrum_snap_specdrum_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
+  /* .z80 format doesn't save the Spectranet state at all */
+  if( libspectrum_snap_spectranet_active( snap ) )
+    *out_flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
+
   error = write_header( buffer, &ptr, length, out_flags, snap );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
@@ -1245,7 +1260,7 @@ write_base_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
     break;
   case LIBSPECTRUM_JOYSTICK_SINCLAIR_2:
     /* Write Z80_JOYSTICK_USER_DEFINED_V3 and custom key mapping corresponding
-       to Sinclair Interface II right port */
+       to Sinclair Interface 2 right port */
     joystick_flags = Z80_JOYSTICK_USER_DEFINED_V3;
     break;
 
@@ -1270,7 +1285,7 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
   int i, bottom_16kb_ram;
   libspectrum_byte hardware_flag = 0;      /* No special emulation features */
   int second_memory_port = 0; /* By default, don't write the extra bytes */
-  libspectrum_byte machine_byte;
+  libspectrum_byte machine_byte = Z80_MACHINE_48;
   size_t header_length;
 
   libspectrum_dword quarter_states;
@@ -1293,6 +1308,7 @@ write_extended_header( libspectrum_byte **buffer, libspectrum_byte **ptr,
     }
     break;
   case LIBSPECTRUM_MACHINE_SE:
+  case LIBSPECTRUM_MACHINE_128E:
     *flags |= LIBSPECTRUM_FLAG_SNAPSHOT_MAJOR_INFO_LOSS;
     /* fall through */
   case LIBSPECTRUM_MACHINE_128:
